/*
 * FILE:
 *   cg_input.cpp
 *
 * AUTHOR:
 *   Stephen Thompson
 *
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2008 - 2009.
 *
 *   This file is part of the "Coercri" software library. Usage of "Coercri"
 *   is permitted under the terms of the Boost Software License, Version 1.0, 
 *   the text of which is displayed below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 */

#include "cg_input.hpp"
#include "../timer/timer.hpp"

namespace Coercri {

    bool CGInput::isKeyQueueEmpty()
    {
        return key_inputs.empty();
    }

    gcn::KeyInput CGInput::dequeueKeyInput()
    {
        if (key_inputs.empty()) {
            throw GCN_EXCEPTION("Key input queue empty");
        }

        gcn::KeyInput k = key_inputs.front();
        key_inputs.pop();
        return k;
    }

    bool CGInput::isMouseQueueEmpty()
    {
        return mouse_inputs.empty();
    }
    
    gcn::MouseInput CGInput::dequeueMouseInput()    
    {
        if (mouse_inputs.empty()) {
            throw GCN_EXCEPTION("Mouse input queue empty");
        }

        gcn::MouseInput m = mouse_inputs.front();
        mouse_inputs.pop();
        return m;
    }

    void CGInput::addKeyInput(const gcn::KeyInput &ki)
    {
        key_inputs.push(ki);
    }

    void CGInput::addMouseInput(const gcn::MouseInput &mi)
    {
        mouse_inputs.push(mi);
    }


    bool CGInput::inputWaiting() const
    {
        return !key_inputs.empty() || !mouse_inputs.empty();
    }
    
}
